@@ -1,6 +1,9 @@
+import argparse
+import asyncio
+import datetime
+import io
 import json
 import logging
-import io
 import os
 import re
 import string
@@ -11,22 +14,13 @@ import uuid
 
 from random import choice
 
-from twisted.internet import reactor, defer
-from twisted.internet.task import deferLater
-from twisted.internet.defer import CancelledError
-from twisted.python import log
-from twisted.web import server, resource, http
-from twisted.web.resource import Resource
-from twisted.web.server import NOT_DONE_YET
-from twisted.web.static import File
+import aiohttp.web as aiohttp_web
 
-from wslink import upload
-
-try:
-    import argparse
-except ImportError:
-    from vtk.util import _argparse as argparse
 
+STATUS_OK = 200
+STATUS_BAD_REQUEST = 400
+STATUS_NOT_FOUND = 404
+STATUS_SERVICE_UNAVAILABLE = 503
 
 sample_config_file = """
 Here is a sample of what a configuration file could look like:
@@ -45,8 +39,7 @@ Here is a sample of what a configuration file could look like:
             "sessionURL" : "ws://${host}:${port}/ws", // ws url used by the client to connect to the started process
             "timeout" : 25,                           // Wait time in second after process start
             "log_dir" : "/.../viz-logs",              // Directory for log files
-            "upload_dir" : "/.../data",               // If launcher should act as upload server, where to put files
-            "fields" : ["file", "host", "port", "updir"]     // List of fields that should be send back to client
+            "fields" : ["file", "host", "port"]       // List of fields that should be send back to client
                                                              // include "secret" if you provide it as an --authKey to the app
             "sanitize": {                             // Check information coming from the client
                 "cmd": {
@@ -69,7 +62,7 @@ Here is a sample of what a configuration file could look like:
         // Useful session vars for client
         // ===============================
 
-        "sessionData" : { "updir": "/Home" },      // Tells client which path to updateFileBrowser after uploads
+        "sessionData" : { "key": "value" },      // Dictionary of values interesting to the client
 
         // ===============================
         // Resources list for applications
@@ -159,6 +152,7 @@ Here is a sample of what a configuration file could look like:
 # Helper module methods
 # =============================================================================
 
+
 def remove_comments(json_like):
     """
     Removes C-style comments from *json_like* and returns the result.  Example::
@@ -175,19 +169,25 @@ def remove_comments(json_like):
     """
     comments_re = re.compile(
         r'//.*?$|/\*.*?\*/|\'(?:\\.|[^\\\'])*\'|"(?:\\.|[^\\"])*"',
-        re.DOTALL | re.MULTILINE
+        re.DOTALL | re.MULTILINE,
     )
+
     def replacer(match):
         s = match.group(0)
-        if s[0] == '/': return ""
+        if s[0] == "/":
+            return ""
         return s
+
     return comments_re.sub(replacer, json_like)
 
+
 def generatePassword():
-    return ''.join(choice(string.ascii_letters + string.digits) for _ in range(16))
+    return "".join(choice(string.ascii_letters + string.digits) for _ in range(16))
+
 
 # -----------------------------------------------------------------------------
 
+
 def validateKeySet(obj, expected_keys, object_name):
     all_key_found = True
     for key in expected_keys:
@@ -196,27 +196,36 @@ def validateKeySet(obj, expected_keys, object_name):
             all_key_found = False
     return all_key_found
 
+
 def checkSanitize(key_pair, sanitize):
     if not sanitize:
-     return
+        return
     for key in sanitize:
         if key in key_pair:
             checkItem = sanitize[key]
             value = key_pair[key]
             if checkItem["type"] == "inList":
                 if not value in checkItem["list"]:
-                    logging.warning("key %s: sanitize %s with default" % (key, key_pair[key]));
+                    logging.warning(
+                        "key %s: sanitize %s with default" % (key, key_pair[key])
+                    )
                     key_pair[key] = checkItem["default"]
             elif checkItem["type"] == "regexp":
                 if not "compiled" in checkItem:
                     # User is responsible to add begin- and end- string symbols, to make sure entire string is matched.
                     checkItem["compiled"] = re.compile(checkItem["regexp"])
                 if checkItem["compiled"].match(value) == None:
-                    logging.warning("key %s: sanitize %s with default" % (key, key_pair[key]));
+                    logging.warning(
+                        "key %s: sanitize %s with default" % (key, key_pair[key])
+                    )
                     key_pair[key] = checkItem["default"]
 
+
 # -----------------------------------------------------------------------------
 # guard against malicious clients - make sure substitution is expected, if 'sanitize' is provided
+# -----------------------------------------------------------------------------
+
+
 def replaceVariables(template_str, variable_list, sanitize):
     for key_pair in variable_list:
         checkSanitize(key_pair, sanitize)
@@ -228,40 +237,49 @@ def replaceVariables(template_str, variable_list, sanitize):
 
     return template_str
 
+
 # -----------------------------------------------------------------------------
 
+
 def replaceList(template_list, variable_list, sanitize):
     result_list = []
     for template_str in template_list:
         result_list.append(replaceVariables(template_str, variable_list, sanitize))
     return result_list
 
+
 # -----------------------------------------------------------------------------
 
+
 def filterResponse(obj, public_keys):
-    public_keys.extend(['id', 'sessionURL', 'sessionManagerURL'])
+    public_keys.extend(["id", "sessionURL", "sessionManagerURL"])
     filtered_output = {}
     for field in obj:
         if field in public_keys:
             filtered_output[field] = obj[field]
     return filtered_output
 
+
 # -----------------------------------------------------------------------------
 
+
 def extractSessionId(request):
-    path = request.path.split('/')
+    path = request.path.split("/")
     if len(path) < 3:
-       return None
+        return None
     return str(path[2])
 
+
 def jsonResponse(payload):
-    return json.dumps(payload, ensure_ascii = False).encode('utf8')
+    return json.dumps(payload, ensure_ascii=False).encode("utf8")
+
+
 # =============================================================================
 # Session manager
 # =============================================================================
 
-class SessionManager(object):
 
+class SessionManager(object):
     def __init__(self, config, mapping):
         self.sessions = {}
         self.config = config
@@ -278,17 +296,29 @@ class SessionManager(object):
 
         # Do we have resources
         if host:
-            options['id'] = id
-            options['host'] = host
-            options['port'] = port
-            if not 'secret' in options:
-                options['secret'] = generatePassword()
-            options['sessionURL'] = replaceVariables(self.config['configuration']['sessionURL'], [options, self.config['properties']], self.sanitize)
-            options['cmd'] = replaceList(self.config['apps'][options['application']]['cmd'], [options, self.config['properties']], self.sanitize)
+            options["id"] = id
+            options["host"] = host
+            options["port"] = port
+            if not "secret" in options:
+                options["secret"] = generatePassword()
+            options["sessionURL"] = replaceVariables(
+                self.config["configuration"]["sessionURL"],
+                [options, self.config["properties"]],
+                self.sanitize,
+            )
+            options["cmd"] = replaceList(
+                self.config["apps"][options["application"]]["cmd"],
+                [options, self.config["properties"]],
+                self.sanitize,
+            )
 
-            if 'sessionData' in self.config :
-                for key in self.config['sessionData'] :
-                    options[key] = replaceVariables(self.config['sessionData'][key], [options, self.config['properties']], self.sanitize)
+            if "sessionData" in self.config:
+                for key in self.config["sessionData"]:
+                    options[key] = replaceVariables(
+                        self.config["sessionData"][key],
+                        [options, self.config["properties"]],
+                        self.sanitize,
+                    )
 
             self.sessions[id] = options
             self.mapping.update(self.sessions)
@@ -297,8 +327,8 @@ class SessionManager(object):
         return None
 
     def deleteSession(self, id):
-        host = self.sessions[id]['host']
-        port = self.sessions[id]['port']
+        host = self.sessions[id]["host"]
+        port = self.sessions[id]["port"]
         self.resources.freeResource(host, port)
         del self.sessions[id]
         self.mapping.update(self.sessions)
@@ -308,17 +338,18 @@ class SessionManager(object):
             return self.sessions[id]
         return None
 
+
 # =============================================================================
 # Proxy manager
 # =============================================================================
 
-class ProxyMappingManager(object):
 
+class ProxyMappingManager(object):
     def update(sessions):
         pass
 
-class ProxyMappingManagerTXT(ProxyMappingManager):
 
+class ProxyMappingManagerTXT(ProxyMappingManager):
     def __init__(self, file_path, pattern="%s %s:%d\n"):
         self.file_path = file_path
         self.pattern = pattern
@@ -326,25 +357,32 @@ class ProxyMappingManagerTXT(ProxyMappingManager):
     def update(self, sessions):
         with io.open(self.file_path, "w", encoding="utf-8") as map_file:
             for id in sessions:
-                map_file.write(self.pattern % (id, sessions[id]['host'], sessions[id]['port']))
+                map_file.write(
+                    self.pattern % (id, sessions[id]["host"], sessions[id]["port"])
+                )
+
 
 # =============================================================================
 # Resource manager
 # =============================================================================
 
+
 class ResourceManager(object):
     """
     Class that provides methods to keep track on available resources (host/port)
     """
+
     def __init__(self, resourceList):
         self.resources = {}
         for resource in resourceList:
-            host = resource['host']
-            portList = list(range(resource['port_range'][0],resource['port_range'][1]+1))
+            host = resource["host"]
+            portList = list(
+                range(resource["port_range"][0], resource["port_range"][1] + 1)
+            )
             if host in self.resources:
-                self.resources[host]['available'].extend(portList)
+                self.resources[host]["available"].extend(portList)
             else:
-                self.resources[host] = { 'available': portList, 'used': []}
+                self.resources[host] = {"available": portList, "used": []}
 
     def getNextResource(self):
         """
@@ -354,13 +392,13 @@ class ResourceManager(object):
         winner = None
         availibilityCount = 0
         for host in self.resources:
-            if availibilityCount < len(self.resources[host]['available']):
-                availibilityCount = len(self.resources[host]['available'])
+            if availibilityCount < len(self.resources[host]["available"]):
+                availibilityCount = len(self.resources[host]["available"])
                 winner = host
 
         if winner:
-            port = self.resources[winner]['available'].pop()
-            self.resources[winner]['used'].append(port)
+            port = self.resources[winner]["available"].pop()
+            self.resources[winner]["used"].append(port)
             return (winner, port)
 
         return (None, None)
@@ -369,19 +407,20 @@ class ResourceManager(object):
         """
         Free a previously reserved resource
         """
-        if host in self.resources and port in self.resources[host]['used']:
-            self.resources[host]['used'].remove(port)
-            self.resources[host]['available'].append(port)
+        if host in self.resources and port in self.resources[host]["used"]:
+            self.resources[host]["used"].remove(port)
+            self.resources[host]["available"].append(port)
 
 
 # =============================================================================
 # Process manager
 # =============================================================================
 
+
 class ProcessManager(object):
     def __init__(self, configuration):
         self.config = configuration
-        self.log_dir = configuration['configuration']['log_dir']
+        self.log_dir = configuration["configuration"]["log_dir"]
         self.processes = {}
 
     def __del__(self):
@@ -395,14 +434,16 @@ class ProcessManager(object):
         proc = None
 
         # Create output log file
-        logFilePath = self._getLogFilePath(session['id'])
+        logFilePath = self._getLogFilePath(session["id"])
         with io.open(logFilePath, mode="a+", buffering=1, encoding="utf-8") as log_file:
             try:
-                proc = subprocess.Popen(session['cmd'], stdout=log_file, stderr=log_file)
-                self.processes[session['id']] = proc
+                proc = subprocess.Popen(
+                    session["cmd"], stdout=log_file, stderr=log_file
+                )
+                self.processes[session["id"]] = proc
             except:
                 logging.error("The command line failed")
-                logging.error(' '.join(map(str, session['cmd'])))
+                logging.error(" ".join(map(str, session["cmd"])))
                 return None
 
         return proc
@@ -413,7 +454,7 @@ class ProcessManager(object):
         try:
             proc.terminate()
         except:
-            pass # we tried
+            pass  # we tried
 
     def listEndedProcess(self):
         session_to_release = []
@@ -431,55 +472,54 @@ class ProcessManager(object):
     # False. This will then rely on the timout time.
     # ========================================================================
 
-    def isReady(self, session, count = 0):
-      id = session['id']
+    def isReady(self, session, count=0):
+        id = session["id"]
 
-      # The process has to be running to be ready!
-      if not self.isRunning(id) and count < 60:
-        return False
+        # The process has to be running to be ready!
+        if not self.isRunning(id) and count < 60:
+            return False
 
-      # Give up after 60 seconds if still not running
-      if not self.isRunning(id):
-        return True
+        # Give up after 60 seconds if still not running
+        if not self.isRunning(id):
+            return True
 
-      application = self.config['apps'][session['application']]
-      ready_line  = application.get('ready_line', None)
+        application = self.config["apps"][session["application"]]
+        ready_line = application.get("ready_line", None)
 
-      # If no ready_line is configured and the process is running then thats
-      # enough.
-      if not ready_line:
-          return False
+        # If no ready_line is configured and the process is running then thats
+        # enough.
+        if not ready_line:
+            return False
 
-      ready = False
+        ready = False
 
-      # Check the output for ready_line
-      logFilePath = self._getLogFilePath(session['id'])
-      with io.open(logFilePath, "r", 1, encoding="utf-8") as log_file:
-          for line in log_file.readlines():
-              if ready_line in line:
-                  ready = True
-                  break
+        # Check the output for ready_line
+        logFilePath = self._getLogFilePath(session["id"])
+        with io.open(logFilePath, "r", 1, encoding="utf-8") as log_file:
+            for line in log_file.readlines():
+                if ready_line in line:
+                    ready = True
+                    break
+
+        return ready
 
-      return ready
 
 # ===========================================================================
 # Class to implement requests to POST, GET and DELETE methods
 # ===========================================================================
 
-class LauncherResource(resource.Resource, object):
+
+class LauncherResource(object):
     def __init__(self, options, config):
-        super(LauncherResource, self).__init__()
         self._options = options
         self._config = config
-        self.time_to_wait = int(config['configuration']['timeout'])
-        self.field_filter = config['configuration']['fields']
-        self.session_manager = SessionManager(config,ProxyMappingManagerTXT(config['configuration']['proxy_file']))
+        self.time_to_wait = int(config["configuration"]["timeout"])
+        self.field_filter = config["configuration"]["fields"]
+        self.session_manager = SessionManager(
+            config, ProxyMappingManagerTXT(config["configuration"]["proxy_file"])
+        )
         self.process_manager = ProcessManager(config)
 
-
-    def getChild(self, path, request):
-        return self
-
     def __del__(self):
         try:
             # causes an exception when server is killed with Ctrl-C
@@ -491,14 +531,14 @@ class LauncherResource(resource.Resource, object):
     # Handle POST request
     # ========================================================================
 
-    def render_POST(self, request):
-        # import ipdb; ipdb.set_trace()
-        payload = json.loads(request.content.getvalue())
+    async def handle_post(self, request):
+        payload = await request.json()
 
         # Make sure the request has all the expected keys
         if not validateKeySet(payload, ["application"], "Launch request"):
-            request.setResponseCode(http.BAD_REQUEST)
-            return jsonResponse({"error": "The request is not complete"})
+            return aiohttp_web.json_response(
+                {"error": "The request is not complete"}, status=STATUS_BAD_REQUEST
+            )
 
         # Try to free any available resource
         id_to_free = self.process_manager.listEndedProcess()
@@ -511,141 +551,112 @@ class LauncherResource(resource.Resource, object):
 
         # No resource available
         if not session:
-            request.setResponseCode(http.SERVICE_UNAVAILABLE)
-            return jsonResponse({"error": "All the resources are currently taken"})
+            return aiohttp_web.json_response(
+                {"error": "All the resources are currently taken"},
+                status=STATUS_SERVICE_UNAVAILABLE,
+            )
 
         # Start process
         proc = self.process_manager.startProcess(session)
 
         if not proc:
-            request.setResponseCode(http.SERVICE_UNAVAILABLE)
-            return jsonResponse({"error": "The process did not properly start. %s" % str(session['cmd'])})
-
-        # local function to act as errback for Deferred objects.
-        def errback(error):
-            # Filter out CancelledError and propagate rest
-            if error.type != CancelledError:
-                return error
-
-        # Deferred object set to timeout request if process doesn't start in time
-        timeout_deferred = deferLater(reactor, self.time_to_wait, lambda: request)
-        timeout_deferred.addCallback(self._delayedRenderTimeout, session)
-        timeout_deferred.addErrback(errback)
-        # Make sure other deferred is canceled once one has been fired
-        request.notifyFinish().addCallback(lambda x: timeout_deferred.cancel())
-
-        # If a ready_line is configured create a Deferred object to wait for
-        # ready line to be produced
-        if 'ready_line' in self._config['apps'][session['application']]:
-            ready_deferred = self._waitForReady(session, request)
-            ready_deferred.addCallback(self._delayedRenderReady, session)
-            ready_deferred.addErrback(errback)
-            # Make sure other deferred is canceled once one has been fired
-            request.notifyFinish().addCallback(lambda x: ready_deferred.cancel())
-
-        return NOT_DONE_YET
+            err_msg = "The process did not properly start. %s" % str(session["cmd"])
+            return aiohttp_web.json_response(
+                {"error": err_msg}, status=STATUS_SERVICE_UNAVAILABLE
+            )
 
+        return await self._waitForReady(session, request)
 
     # ========================================================================
-    # Wait for session to be ready. Rather than blocking keep using callLater(...)
-    # to schedule self in reactor. Return a Deferred object whose callback will
-    # be triggered when the session is ready
+    # Wait for session to be ready
     # ========================================================================
 
-    def _waitForReady(self, session, request, count=0, d=None):
-        if not d:
-            d = defer.Deferred()
+    async def _waitForReady(self, session, request):
+        start_time = datetime.datetime.now()
+        check_line = "ready_line" in self._config["apps"][session["application"]]
+        count = 0
 
-        if not 'startTimedOut' in session and \
-            not self.process_manager.isReady(session, count + 1):
-            reactor.callLater(1, self._waitForReady, session, request, count + 1, d)
-        else:
-            d.callback(request)
+        while True:
+            if self.process_manager.isReady(session, count):
+                filterkeys = self.field_filter
+                if session["secret"] in session["cmd"]:
+                    filterkeys = self.field_filter + ["secret"]
+                return aiohttp_web.json_response(
+                    filterResponse(session, filterkeys), status=STATUS_OK
+                )
 
-        return d
+            elapsed_time = datetime.datetime.now() - start_time
 
-    # ========================================================================
-    # Called when the timeout out expires. Check if process is now ready
-    # and send response to client.
-    # ========================================================================
+            if elapsed_time.total_seconds() > self.time_to_wait:
+                # Timeout is expired, if the process is not ready now, mark the
+                # session as timed out, clean up the process, and return an error
+                # response
+                session["startTimedOut"] = True
+                self.session_manager.deleteSession(session["id"])
+                self.process_manager.stopProcess(session["id"])
 
-    def _delayedRenderTimeout(self, request, session):
-        ready = self.process_manager.isReady(session, 0)
+                return aiohttp_web.json_response(
+                    {
+                        "error": "Session did not start before timeout expired. Check session logs."
+                    },
+                    status=STATUS_SERVICE_UNAVAILABLE,
+                )
 
-        if ready:
-            request.write(jsonResponse(filterResponse(session, self.field_filter)))
-            request.setResponseCode(http.OK)
-        else:
-            request.write(jsonResponse({"error": "Session did not start before timeout expired. Check session logs."}))
-            # Mark the session as timed out and clean up the process
-            session['startTimedOut'] = True
-            self.session_manager.deleteSession(session['id'])
-            self.process_manager.stopProcess(session['id'])
-            request.setResponseCode(http.SERVICE_UNAVAILABLE)
-
-        request.finish()
-
-    # ========================================================================
-    # Called when the process is ready ( the ready line has been read from the
-    # process output).
-    # ========================================================================
-
-    def _delayedRenderReady(self, request, session):
-        filterkeys = self.field_filter
-        if session['secret'] in session['cmd']:
-            filterkeys = self.field_filter + [ 'secret' ]
-        request.write(jsonResponse(filterResponse(session, filterkeys)))
-        request.setResponseCode(http.OK)
-
-        request.finish()
+            await asyncio.sleep(1)
+            count += 1
 
     # =========================================================================
     # Handle GET request
     # =========================================================================
 
-    def render_GET(self, request):
+    async def handle_get(self, request):
         id = extractSessionId(request)
 
         if not id:
-           message = "id not provided in GET request"
-           logging.error(message)
-           request.setResponseCode(http.BAD_REQUEST)
-           return jsonResponse({"error":message})
+            message = "id not provided in GET request"
+            logging.error(message)
+            return aiohttp_web.json_response(
+                {"error": message}, status=STATUS_BAD_REQUEST
+            )
 
         logging.info("GET request received for id: %s" % id)
 
         session = self.session_manager.getSession(id)
         if not session:
-           message = "No session with id: %s" % id
-           logging.error(message)
-           request.setResponseCode(http.NOT_FOUND)
-           return jsonResponse({"error":message})
+            message = "No session with id: %s" % id
+            logging.error(message)
+            return aiohttp_web.json_response(
+                {"error": message}, status=STATUS_BAD_REQUEST
+            )
 
         # Return session meta-data
-        request.setResponseCode(http.OK)
-        return jsonResponse(filterResponse(session, self.field_filter))
+        return aiohttp_web.json_response(
+            filterResponse(session, self.field_filter), status=STATUS_OK
+        )
 
     # =========================================================================
     # Handle DELETE request
     # =========================================================================
 
-    def render_DELETE(self, request):
+    async def handle_delete(self, request):
         id = extractSessionId(request)
 
         if not id:
-           message = "id not provided in DELETE request"
-           logging.error(message)
-           request.setResponseCode(http.BAD_REQUEST)
-           return jsonResponse({"error":message})
+            message = "id not provided in DELETE request"
+            logging.error(message)
+            return aiohttp_web.json_response(
+                {"error": message}, status=STATUS_BAD_REQUEST
+            )
 
         logging.info("DELETE request received for id: %s" % id)
 
         session = self.session_manager.getSession(id)
         if not session:
-           message = "No session with id: %s" % id
-           logging.error(message)
-           request.setResponseCode(http.NOT_FOUND)
-           return jsonResponse({"error":message})
+            message = "No session with id: %s" % id
+            logging.error(message)
+            return aiohttp_web.json_response(
+                {"error": message}, status=STATUS_NOT_FOUND
+            )
 
         # Remove session
         self.session_manager.deleteSession(id)
@@ -654,62 +665,69 @@ class LauncherResource(resource.Resource, object):
         message = "Deleted session with id: %s" % id
         logging.info(message)
 
-        request.setResponseCode(http.OK)
-        return session
+        return aiohttp_web.json_response(session, status=STATUS_OK)
+
 
 # =============================================================================
 # Start the web server
 # =============================================================================
 
+
 def startWebServer(options, config):
     # import pdb; pdb.set_trace()
     # Extract properties from config
-    log_dir  = str(config["configuration"]["log_dir"])
-    content  = str(config["configuration"]["content"])
-    # twisted expects byte for uris.
-    endpoint = str(config["configuration"]["endpoint"]).encode('utf-8')
-    host     = str(config["configuration"]["host"])
-    port     = int(config["configuration"]["port"])
+    log_dir = str(config["configuration"]["log_dir"])
+    content = str(config["configuration"]["content"])
+    endpoint = str(config["configuration"]["endpoint"])
+    host = str(config["configuration"]["host"])
+    port = int(config["configuration"]["port"])
     sanitize = config["configuration"]["sanitize"]
 
     # Setup logging
     logFileName = log_dir + os.sep + "launcherLog.log"
-    formatting = '%(asctime)s:%(levelname)s:%(name)s:%(message)s'
-    logging.basicConfig(level=logging.DEBUG, filename=logFileName, filemode='w', format=formatting)
-    observer = log.PythonLoggingObserver()
-    observer.start()
+    formatting = "%(asctime)s:%(levelname)s:%(name)s:%(message)s"
+    logging.basicConfig(
+        level=logging.DEBUG, filename=logFileName, filemode="w", format=formatting
+    )
     if options.debug:
         console = logging.StreamHandler(sys.stdout)
         console.setLevel(logging.INFO)
         formatter = logging.Formatter(formatting)
         console.setFormatter(formatter)
-        logging.getLogger('').addHandler(console)
+        logging.getLogger("").addHandler(console)
 
-    # Initialize web resource
-    web_resource = File(content) if (len(content) > 0) else resource.Resource()
+    web_app = aiohttp_web.Application()
 
-    # Attach launcher
-    web_resource.putChild(endpoint, LauncherResource(options, config))
+    launcher_resource = LauncherResource(options, config)
 
-    # Check if launcher should act as a file upload server as well
-    if "upload_dir" in config["configuration"]:
-        from upload import UploadPage
-        updir = replaceVariables(config['configuration']['upload_dir'], [config['properties']], sanitize)
-        uploadResource = UploadPage(updir)
-        web_resource.putChild("upload", uploadResource)
+    if not endpoint.startswith("/"):
+        endpoint = "/{0}/".format(endpoint)
+
+    web_app.add_routes(
+        [
+            aiohttp_web.post(endpoint, launcher_resource.handle_post),
+            aiohttp_web.get(endpoint, launcher_resource.handle_get),
+            aiohttp_web.delete(endpoint, launcher_resource.handle_delete),
+        ]
+    )
+
+    if len(content) > 0:
+        web_app.add_routes([aiohttp_web.static("/", content)])
+
+    aiohttp_web.run_app(web_app, host=host, port=port)
 
-    site = server.Site(web_resource)
-    reactor.listenTCP(port, site, interface=host)
-    reactor.run()
 
 # =============================================================================
 # Parse config file
 # =============================================================================
 
+
 def parseConfig(options):
     # Read values from the configuration file
     try:
-        config_comments = remove_comments(io.open(options.config[0], encoding="utf-8").read())
+        config_comments = remove_comments(
+            io.open(options.config[0], encoding="utf-8").read()
+        )
         config = json.loads(config_comments)
     except:
         message = "ERROR: Unable to read config file.\n"
@@ -723,7 +741,16 @@ def parseConfig(options):
         print(sample_config_file)
         sys.exit(2)
 
-    expected_keys = ["endpoint", "host", "port", "proxy_file", "sessionURL", "timeout", "log_dir", "fields"]
+    expected_keys = [
+        "endpoint",
+        "host",
+        "port",
+        "proxy_file",
+        "sessionURL",
+        "timeout",
+        "log_dir",
+        "fields",
+    ]
     if not validateKeySet(config["configuration"], expected_keys, "file.configuration"):
         print(sample_config_file)
         sys.exit(2)
@@ -735,36 +762,42 @@ def parseConfig(options):
 
     return config
 
+
 # =============================================================================
 # Setup default arguments to be parsed
 #   -d, --debug
 #   -t, --proxyFileType  Type of proxy file (txt, dbm)
 # =============================================================================
 
+
 def add_arguments(parser):
-    parser.add_argument("config", type=str,  nargs=1,
-        help="configuration file for the launcher")
-    parser.add_argument("-d", "--debug",
-        help="log debugging messages to stdout",
-        action="store_true")
+    parser.add_argument(
+        "config", type=str, nargs=1, help="configuration file for the launcher"
+    )
+    parser.add_argument(
+        "-d", "--debug", help="log debugging messages to stdout", action="store_true"
+    )
 
     return parser
 
+
 # =============================================================================
 # Parse arguments
 # =============================================================================
 
-def start(argv=None,
-         description="wslink Web Launcher"):
+
+def start(argv=None, description="wslink Web Launcher"):
     parser = argparse.ArgumentParser(
         description=description,
         formatter_class=argparse.RawDescriptionHelpFormatter,
-        epilog=sample_config_file)
+        epilog=sample_config_file,
+    )
     add_arguments(parser)
     args = parser.parse_args(argv)
     config = parseConfig(args)
     startWebServer(args, config)
 
+
 # =============================================================================
 # Main
 # =============================================================================
